# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# Authored by: Martin Borho <martin@borho.net>

"""weather-app autopilot tests."""

import os.path
import glob
import sqlite3
import time

from autopilot.input import Mouse, Touch, Pointer
from autopilot.platform import model
from autopilot.testcase import AutopilotTestCase
from testtools.matchers import Equals, NotEquals, Is, Not
from autopilot.matchers import Eventually

from ubuntuuitoolkit import emulators as toolkit_emulators
from ubuntu_weather_app.emulators import MainView


class WeatherTestCase(AutopilotTestCase):

    """A common test case class that provides several useful methods for
       weather app tests."""

    if model() == 'Desktop':
        scenarios = [('with mouse', dict(input_device_class=Mouse))]
    else:
        scenarios = [('with touch', dict(input_device_class=Touch))]

    local_location = "../../ubuntu-weather-app.qml"
    installed_location = "/usr/share/ubuntu-weather-app/ubuntu-weather-app.qml"

    def setUp(self):
        self.pointing_device = Pointer(self.input_device_class.create())
        super(WeatherTestCase, self).setUp()
        self.launch_app()

    """Workaround to find the qmlscene binary via shell globbing.
       This is needed since we can't rely on qt5-default being installed on
       devices to make qmlscene available in the path"""
    def qmlscene(self):
        return glob.glob("/usr/lib/*/qt5/bin/qmlscene")[0]

    def launch_app(self):
        if os.path.exists(self.local_location):
            self.launch_test_local()
        elif os.path.exists(self.installed_location):
            self.launch_test_installed()
        else:
            self.launch_test_click()

    def launch_test_local(self):
        self.app = self.launch_test_application(
            self.qmlscene(),
            self.local_location,
            app_type='qt',
            emulator_base=toolkit_emulators.UbuntuUIToolkitEmulatorBase)

    def launch_test_installed(self):
        self.app = self.launch_test_application(
            self.qmlscene(),
            self.installed_location,
            "--desktop_file_hint=/usr/share/applications/"
            "ubuntu-weather-app.desktop",
            app_type='qt',
            emulator_base=toolkit_emulators.UbuntuUIToolkitEmulatorBase)

    def launch_test_click(self):
        self.app = self.launch_click_package(
            "com.ubuntu.weather",
            emulator_base=toolkit_emulators.UbuntuUIToolkitEmulatorBase)

    def launch_and_quit_app(self):
        self.launch_app()
        self.get_qml_view().visible.wait_for(True)

        # When calling launch_app an instance of the spawned process
        # control object will be stored in self.app.process, and a cleanup
        # handler will be registered that essentially kills the process.
        # Therefore, by triggering the cleanup handler here we're killing the
        # process and removing the handler, which allows a clean launch of
        # the process during regular test setup.
        self.doCleanups()

    def get_qml_view(self):
        return self.app.select_single("QQuickView")

    @property
    def main_window(self):
        return self.app.select_single(MainView)


class SheetMixin(object):
    """A mixin to to give access to common sheet elements"""

    def _click_sheet_confirm(self):
        """Clicks the confirm button"""
        button = self.main_window.select_single(
            'Button', objectName='confirmButton')
        self.pointing_device.click_object(button)

    def _click_sheet_cancel(self):
        """Clicks the cancel button"""
        button = self.main_window.select_single(
            'Button', objectName='cancelButton')
        self.pointing_device.click_object(button)


class LocationManagerMixin(object):
    """Mixin for LocationManager related methods"""
    def _open_add_location_page(self):
        """Opens the AddLocationPage"""
        # click on listitem to open AddLocationPage
        self.assertThat(lambda: self.main_window.select_single('Standard', objectName='AddCityListItem'), Eventually(Not(Is(None))))
        addCityItem = self.main_window.select_single('Standard', objectName='AddCityListItem')
        self.assertThat(lambda: addCityItem.visible, Eventually(Equals(True)))
        self.pointing_device.move_to_object(addCityItem)
        self.pointing_device.click()

        addLocPage = self.main_window.select_single("DefaultSheet", objectName="AddLocationSheet")
        self.assertThat(addLocPage.visible, Eventually(Equals(True)))

class DatabaseMixin(object):

    """
    Helper functions for dealing with sqlite databases
    """

    def find_db(self):
        dbs_path = os.path.expanduser(
            "~/.local/share/com.ubuntu.weather/Databases/")
        if not os.path.exists(dbs_path):
            return None

        files = [f for f in os.listdir(dbs_path)
                 if os.path.splitext(f)[1] == ".ini"]
        for f in files:
            ini_path = os.path.join(dbs_path, f)
            with open(ini_path) as ini:
                for line in ini:
                    if "=" in line:
                        key, val = line.strip().split("=")
                        if key == "Name" and val == "com.ubuntu.weather":
                            try:
                                return ini_path.replace(".ini", ".sqlite")
                            except OSError:
                                pass
        return None

    def clean_db(self):
        path = self.find_db()
        if path is None:
            self.launch_and_quit_app()
            path = self.find_db()
            if path is None:
                self.assertNotEquals(path, None)

        try:
            os.remove(path)
        except OSError:
            pass

    def save_locations_to_storage(self, locations):
        path = self.find_db()
        conn = sqlite3.connect(path)
        cursor = conn.cursor()
        for loc_data in locations:
            db_exec = "INSERT INTO Locations(date, data) VALUES('{}', '{}')"
            cursor.execute(db_exec.format(
                int(time.time()*1000), loc_data))
            conn.commit()
        conn.close()

    def save_units_to_storage(self):
        path = self.find_db()
        conn = sqlite3.connect(path)
        cursor = conn.cursor()
        cursor.execute("INSERT INTO settings(key, value) "
                       "VALUES('units', 'metric'), "
                       "('wind_units', 'kmh'), "
                       "('precip_units', 'mm')")
        conn.commit()
        conn.close()
